# coding: utf-8
import unittest
import io

from ecoji import decode


class TestDecode(unittest.TestCase):
    def test_decode(self):
        r = io.StringIO('🎌🚟🎗🈸🎥🤠📠🐂🎐🚯🏛🐇\n')
        w = io.BytesIO()
        decode(r, w)
        self.assertEqual(b'123456789123456', w.getvalue())
        r.close()
        w.close()



# coding: utf-8
import unittest
import io

from ecoji import encode


class TestEncode(unittest.TestCase):
    def test_readnum_equals5(self):
        r = io.BytesIO(b'12345')
        w = io.StringIO()
        encode(r, w)
        self.assertEqual('🎌🚟🎗🈸\n', w.getvalue())
        r.close()
        w.close()

    def test_readnum_equals4(self):
        r = io.BytesIO(b'123456789')
        w = io.StringIO()
        encode(r, w)
        self.assertEqual('🎌🚟🎗🈸🎥🤠📠🏍\n', w.getvalue())
        r.close()
        w.close()

    def test_readnum_equals3(self):
        r = io.BytesIO(b'12345678')
        w = io.StringIO()
        encode(r, w)
        self.assertEqual('🎌🚟🎗🈸🎥🤠📒☕\n', w.getvalue())
        r.close()
        w.close()

    def test_readnum_equals2(self):
        r = io.BytesIO(b'1234567')
        w = io.StringIO()
        encode(r, w)
        self.assertEqual('🎌🚟🎗🈸🎥🤝☕☕\n', w.getvalue())
        r.close()
        w.close()

    def test_readnum_equals1(self):
        r = io.BytesIO(b'123456')
        w = io.StringIO()
        encode(r, w)
        self.assertEqual('🎌🚟🎗🈸🎥☕☕☕\n', w.getvalue())
        r.close()
        w.close()

    def test_wrap_cols(self):
        r = io.BytesIO(b'123456789123456')
        w = io.StringIO()
        encode(r, w, 6)
        wanted_str = '🎌🚟🎗🈸🎥🤠📠🐂\n🎐🚯🏛🐇\n'
        self.assertEqual(wanted_str, w.getvalue())
        r.close()
        w.close()

    def test_negative_wrap_cols(self):
        r = io.BytesIO(b'123456789123456')
        w = io.StringIO()
        encode(r, w, -1)
        wanted_str = '🎌🚟🎗🈸🎥🤠📠🐂🎐🚯🏛🐇\n'
        self.assertEqual(wanted_str, w.getvalue())
        r.close()
        w.close()



# coding: utf-8
"""usage: ecoji [OPTIONS]... [FILE]

Encode or decode data as Unicode emojis. 😁

Options:
    -d, --decode          decode data
    -w, --wrap=COLS       wrap encoded lines after COLS character (default 76).
                          Use 0 to disable line wrapping
    -h, --help            Print this message
    -v, --version         Print version information.
"""
import sys
import argparse
from . import encode, decode, __version__, _DEFAULT_WRAP_COLS

_VERSION_MSG = """Ecoji-py version 1.0.0
    Copyright   : (C) 2018 mecforlove
    License     : Apache 2.0
    Source code : https://github.com/mecforlove/ecoji-py
"""


def main():
    parser = argparse.ArgumentParser()
    parser.usage = __doc__
    parser.add_argument(
        '-v',
        '--version',
        help='print version information',
        action='store_true')
    parser.add_argument(
        '-w', '--wrap', help='cols to wrap encoded lines', type=int)
    parser.add_argument(
        'filename', nargs='?', help='filename to encode', action='store')
    parser.add_argument(
        '-d', '--decode', help='decode data', action='store_true')
    args = parser.parse_args()
    if args.version:
        print(_VERSION_MSG)
    if args.decode:
        if args.filename:
            decode(open(args.filename[0], 'r'), sys.stdout.buffer)
        else:
            decode(sys.stdin, sys.stdout.buffer)
    if args.wrap is not None:
        wrap = wrap
    else:
        wrap = _DEFAULT_WRAP_COLS
    if args.filename:
        encode(open(args.filename[0], 'rb'), sys.stdout, wrap)
    else:
        encode(sys.stdin.buffer, sys.stdout, wrap)


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

__version__ = '0.1.1'

setup(
    name='ecoji',
    version=__version__,
    description='Encode and decode data as emojis.',
    long_description=open("README.md", encoding="utf-8").read(),
    author='mecforlove',
    author_email='mecforlove@outlook.com',
    url='https://github.com/mecforlove/ecoji-py',
    license='Apache',
    packages=find_packages(),
    entry_points='''
        [console_scripts]
        ecoji=ecoji.__main__:main
    ''',
    classifiers=[
        'Programming Language :: Python', 'Programming Language :: Python :: 3'
    ])



# coding: utf-8
from .mapping import EMOJIS, PADDING, PADDING40, PADDING41, PADDING42, PADDING43, EMOJI2BYTE

__version__ = '0.1.0'
__all__ = ('encode', 'decode')

_DEFAULT_WRAP_COLS = 76


class EcojiError(Exception):
    pass


class DecodeError(EcojiError):
    pass


def encode(reader, writer, wrap=_DEFAULT_WRAP_COLS):
    read_bytes = reader.read(5)
    num_read = len(read_bytes)
    written_len = 0
    while num_read:
        data = [0] * 5
        for index, byte in enumerate(read_bytes):
            data[index] = byte

        if num_read == 1:
            writer.write(EMOJIS[data[0] << 2 | data[1] >> 6])
            writer.write(PADDING)
            writer.write(PADDING)
            writer.write(PADDING)
        elif num_read == 2:
            writer.write(EMOJIS[data[0] << 2 | data[1] >> 6])
            writer.write(EMOJIS[(data[1] & 0x3f) << 4 | data[2] >> 4])
            writer.write(PADDING)
            writer.write(PADDING)
        elif num_read == 3:
            writer.write(EMOJIS[data[0] << 2 | data[1] >> 6])
            writer.write(EMOJIS[(data[1] & 0x3f) << 4 | data[2] >> 4])
            writer.write(EMOJIS[(data[2] & 0x0f) << 6 | data[3] >> 2])
            writer.write(PADDING)
        elif num_read == 4:
            writer.write(EMOJIS[data[0] << 2 | data[1] >> 6])
            writer.write(EMOJIS[(data[1] & 0x3f) << 4 | data[2] >> 4])
            writer.write(EMOJIS[(data[2] & 0x0f) << 6 | data[3] >> 2])
            t = data[3] & 0x03
            if t == 0:
                writer.write(PADDING40)
            if t == 1:
                writer.write(PADDING41)
            if t == 2:
                writer.write(PADDING42)
            if t == 3:
                writer.write(PADDING43)
        else:
            writer.write(EMOJIS[data[0] << 2 | data[1] >> 6])
            writer.write(EMOJIS[(data[1] & 0x3f) << 4 | data[2] >> 4])
            writer.write(EMOJIS[(data[2] & 0x0f) << 6 | data[3] >> 2])
            writer.write(EMOJIS[(data[3] & 0x03) << 8 | data[4]])
        written_len += 4
        if wrap > 0:
            if written_len >= wrap:
                writer.write('\n')
                written_len = 0
        read_bytes = reader.read(5)
        num_read = len(read_bytes)
    if written_len:
        writer.write('\n')


def decode(reader, writer):
    char0 = _read_emoji(reader)
    while len(char0):
        char1 = _read_emoji(reader)
        char2 = _read_emoji(reader)
        char3 = _read_emoji(reader)

        bits0 = EMOJI2BYTE.get(char0) or 0
        bits1 = EMOJI2BYTE.get(char1) or 0
        bits2 = EMOJI2BYTE.get(char2) or 0
        if char3 == PADDING40:
            bits3 = 0
        if char3 == PADDING41:
            bits3 = 1 << 8
        if char3 == PADDING42:
            bits3 = 2 << 8
        if char3 == PADDING43:
            bits3 = 3 << 8
        else:
            bits3 = EMOJI2BYTE.get(char3) or 0
        out = [0] * 5
        out[0] = bits0 >> 2
        out[1] = ((bits0 & 0x3) << 6) | (bits1 >> 4)
        out[2] = ((bits1 & 0xf) << 4) | (bits2 >> 6)
        out[3] = ((bits2 & 0x3f) << 2) | (bits3 >> 8)
        out[4] = bits3 & 0xff
        out = bytes(out)
        if char1 == PADDING:
            out = out[:1]
        if char2 == PADDING:
            out = out[:2]
        if char3 == PADDING:
            out = out[:3]
        if char3 in (PADDING40, PADDING41, PADDING42, PADDING43):
            out = out[:4]
        writer.write(out)
        char0 = _read_emoji(reader)


def _read_emoji(reader):
    c = reader.read(1)
    if len(c) == 0:
        return ''
    while c == '\n':
        c = reader.read(1)
    return c
